# This file has been autogenerated by the pywayland scanner

# Copyright 2022 Simon Ser
# Copyright 2022 Red Hat, Inc.
# Copyright 2022 Collabora, Ltd.
# Copyright 2022-2025 Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpColorRepresentationManagerV1(Interface):
    """Color representation manager singleton

    A singleton global interface used for getting color representation
    extensions for :class:`~pywayland.protocol.wayland.WlSurface`. The
    extension interfaces allow setting the color representation of surfaces.

    Compositors should never remove this global.
    """

    name = "wp_color_representation_manager_v1"
    version = 1

    class error(enum.IntEnum):
        surface_exists = 1


class WpColorRepresentationSurfaceV1(Interface):
    """Color representation extension to a surface

    A :class:`WpColorRepresentationSurfaceV1` allows the client to set the
    color representation metadata of a surface.

    By default, a surface does not have any color representation metadata set.
    The reconstruction of R, G, B signals on such surfaces is compositor
    implementation defined. The alpha mode is assumed to be
    premultiplied_electrical when the alpha mode is unset.

    If the :class:`~pywayland.protocol.wayland.WlSurface` associated with the
    :class:`WpColorRepresentationSurfaceV1` is destroyed, the
    :class:`WpColorRepresentationSurfaceV1` object becomes inert.
    """

    name = "wp_color_representation_surface_v1"
    version = 1

    class error(enum.IntEnum):
        alpha_mode = 1
        coefficients = 2
        pixel_format = 3
        inert = 4
        chroma_location = 5

    class alpha_mode(enum.IntEnum):
        premultiplied_electrical = 0
        premultiplied_optical = 1
        straight = 2

    class coefficients(enum.IntEnum):
        identity = 1
        bt709 = 2
        fcc = 3
        bt601 = 4
        smpte240 = 5
        bt2020 = 6
        bt2020_cl = 7
        ictcp = 8

    class range(enum.IntEnum):
        full = 1
        limited = 2

    class chroma_location(enum.IntEnum):
        type_0 = 1
        type_1 = 2
        type_2 = 3
        type_3 = 4
        type_4 = 5
        type_5 = 6


class WpColorRepresentationManagerV1Resource(Resource[WpColorRepresentationManagerV1]):
    interface = WpColorRepresentationManagerV1

    @WpColorRepresentationManagerV1.event(
        Argument(ArgumentType.Uint),
    )
    def supported_alpha_mode(self, alpha_mode: int) -> None:
        """Supported alpha modes

        When this object is created, it shall immediately send this event once
        for each alpha mode the compositor supports.

        For the definition of the supported values, see the
        :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`::alpha_mode
        enum.

        :param alpha_mode:
            supported alpha mode
        :type alpha_mode:
            `ArgumentType.Uint`
        """
        self._post_event(0, alpha_mode)

    @WpColorRepresentationManagerV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def supported_coefficients_and_ranges(self, coefficients: int, range: int) -> None:
        """Supported matrix coefficients and ranges

        When this object is created, it shall immediately send this event once
        for each matrix coefficient and color range combination the compositor
        supports.

        For the definition of the supported values, see the
        :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`::coefficients
        and
        :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`::range
        enums.

        :param coefficients:
            supported matrix coefficients
        :type coefficients:
            `ArgumentType.Uint`
        :param range:
            full range flag
        :type range:
            `ArgumentType.Uint`
        """
        self._post_event(1, coefficients, range)

    @WpColorRepresentationManagerV1.event()
    def done(self) -> None:
        """All features have been sent

        This event is sent when all supported features have been sent.
        """
        self._post_event(2)


class WpColorRepresentationSurfaceV1Resource(Resource[WpColorRepresentationSurfaceV1]):
    interface = WpColorRepresentationSurfaceV1


class WpColorRepresentationManagerV1Proxy(Proxy[WpColorRepresentationManagerV1]):
    interface = WpColorRepresentationManagerV1

    @WpColorRepresentationManagerV1.request()
    def destroy(self) -> None:
        """Destroy the manager

        Destroy the :class:`WpColorRepresentationManagerV1` object. This does
        not affect any other objects in any way.
        """
        self._marshal(0)
        self._destroy()

    @WpColorRepresentationManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpColorRepresentationSurfaceV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface(self, surface: WlSurfaceProxy) -> WpColorRepresentationSurfaceV1Proxy:
        """Create a color representation interface for a :class:`~pywayland.protocol.wayland.WlSurface`

        If a
        :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`
        object already exists for the given
        :class:`~pywayland.protocol.wayland.WlSurface`, the protocol error
        surface_exists is raised.

        This creates a new color
        :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`
        object for the given :class:`~pywayland.protocol.wayland.WlSurface`.

        See the
        :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`
        interface for more details.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.color_representation_v1.WpColorRepresentationSurfaceV1`
        """
        id = self._marshal_constructor(1, WpColorRepresentationSurfaceV1, surface)
        return id


class WpColorRepresentationSurfaceV1Proxy(Proxy[WpColorRepresentationSurfaceV1]):
    interface = WpColorRepresentationSurfaceV1

    @WpColorRepresentationSurfaceV1.request()
    def destroy(self) -> None:
        """Destroy the color representation

        Destroy the :class:`WpColorRepresentationSurfaceV1` object.

        Destroying this object unsets all the color representation metadata
        from the surface. See the :class:`WpColorRepresentationSurfaceV1`
        interface description for how a compositor handles a surface without
        color representation metadata. Unsetting is double-buffered state, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.
        """
        self._marshal(0)
        self._destroy()

    @WpColorRepresentationSurfaceV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_alpha_mode(self, alpha_mode: int) -> None:
        """Set the surface alpha mode

        If this protocol object is inert, the protocol error inert is raised.

        Assuming an alpha channel exists, it is always linear. The alpha mode
        determines whether and how the color channels include pre-multiplied
        alpha. Using straight alpha might have performance benefits.

        Only alpha modes advertised by the compositor are allowed to be used as
        argument for this request. The "alpha_mode" protocol error is raised
        otherwise.

        Alpha mode is double buffered, see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        :param alpha_mode:
            alpha mode
        :type alpha_mode:
            `ArgumentType.Uint`
        """
        self._marshal(1, alpha_mode)

    @WpColorRepresentationSurfaceV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_coefficients_and_range(self, coefficients: int, range: int) -> None:
        """Set the matrix coefficients and range

        If this protocol object is inert, the protocol error inert is raised.

        Set the matrix coefficients and video range which defines the formula
        and the related constants used to derive red, green and blue signals.
        Usually coefficients correspond to MatrixCoefficients code points in
        H.273.

        Only combinations advertised by the compositor are allowed to be used
        as argument for this request. The "coefficients" protocol error is
        raised otherwise.

        A call to :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` verifies that the pixel
        format and the coefficients-range combination in the committed surface
        contents are compatible, if contents exist. The "pixel_format" protocol
        error is raised otherwise.

        A pixel format is compatible with the coefficients-range combination if
        the related equations and conventions as defined in H.273 can produce
        the color channels (RGB or YCbCr) of the pixel format.

        For the definition of the supported combination, see the
        :class:`WpColorRepresentationSurfaceV1`::coefficients and
        :class:`WpColorRepresentationSurfaceV1`::range enums.

        The coefficients-range combination is double-buffered, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        :param coefficients:
            matrix coefficients
        :type coefficients:
            `ArgumentType.Uint`
        :param range:
            range
        :type range:
            `ArgumentType.Uint`
        """
        self._marshal(2, coefficients, range)

    @WpColorRepresentationSurfaceV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_chroma_location(self, chroma_location: int) -> None:
        """Set the chroma location

        If this protocol object is inert, the protocol error inert is raised.

        Set the chroma location type which defines the position of downsampled
        chroma samples, corresponding to Chroma420SampleLocType code points in
        H.273.

        An invalid chroma location enum value raises the "chroma_location"
        protocol error.

        A call to :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` verifies that the pixel
        format and chroma location type in the committed surface contents are
        compatible, if contents exist. The "pixel_format" protocol error is
        raised otherwise.

        For the definition of the supported chroma location types, see the
        :class:`WpColorRepresentationSurfaceV1`::chroma_location enum.

        The chroma location type is double-buffered, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        :param chroma_location:
            chroma sample location
        :type chroma_location:
            `ArgumentType.Uint`
        """
        self._marshal(3, chroma_location)


class WpColorRepresentationManagerV1Global(Global[WpColorRepresentationManagerV1]):
    interface = WpColorRepresentationManagerV1


class WpColorRepresentationSurfaceV1Global(Global[WpColorRepresentationSurfaceV1]):
    interface = WpColorRepresentationSurfaceV1


WpColorRepresentationManagerV1._gen_c()
WpColorRepresentationManagerV1.proxy_class = WpColorRepresentationManagerV1Proxy
WpColorRepresentationManagerV1.resource_class = WpColorRepresentationManagerV1Resource
WpColorRepresentationManagerV1.global_class = WpColorRepresentationManagerV1Global


WpColorRepresentationSurfaceV1._gen_c()
WpColorRepresentationSurfaceV1.proxy_class = WpColorRepresentationSurfaceV1Proxy
WpColorRepresentationSurfaceV1.resource_class = WpColorRepresentationSurfaceV1Resource
WpColorRepresentationSurfaceV1.global_class = WpColorRepresentationSurfaceV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from typing import TYPE_CHECKING
from weakref import WeakKeyDictionary, WeakValueDictionary

from pywayland import ffi

from .message import Message

if TYPE_CHECKING:
    from collections.abc import Callable
    from typing import Any, ParamSpec, TypeVar

    from .argument import Argument
    from .globals import Global
    from .proxy import Proxy
    from .resource import Resource

    P = ParamSpec("P")
    R = TypeVar("R")

weakkeydict: WeakKeyDictionary[ffi.WlInterface, tuple[object, ...]] = (
    WeakKeyDictionary()
)


class InterfaceMeta(type):
    """Metaclass for Interfaces

    Initializes empty lists for events and requests for the given class.
    """

    def __init__(self, name: str, bases: tuple[Any], dct: dict[Any, Any]):
        self.events: list[Message] = []
        self.requests: list[Message] = []

        # Initialize the interface cdata
        self._ptr: ffi.WlInterface = ffi.new("struct wl_interface *")


class Interface(metaclass=InterfaceMeta):
    """Wrapper class for wl_wayland structs

    Base class for interfaces that are defined by the wayland.xml class and
    generated by the scanner.  Sub-classes should use the
    :class:`InterfaceMeta` metaclass, which will define subclass.events and
    subclass.requests, the lists of the methods on this interface.  These class
    variables are populated by the :func:`Interface.event` and
    :func:`Interface.request` decorators.
    """

    _ptr: ffi.WlInterface
    name: str
    version: int
    proxy_class: type[Proxy[Any]]
    resource_class: type[Resource[Any]]
    global_class: type[Global[Any]]
    registry: WeakValueDictionary[ffi.WlProxy, Proxy[Any]]

    @classmethod
    def event(
        cls, *arguments: Argument, version: int | None = None
    ) -> Callable[[Callable[P, R]], Callable[P, R]]:
        """Decorator for interface events

        Adds the decorated method to the list of events of the interface
        (server-side method).

        :param arguments: Arguments for the event
        :type arguments: :class:`~pywayland.protocol_core.Argument`
        :param version: Version of the event (optional)
        :type version: int or None
        """

        def wrapper(func: Callable[P, R]) -> Callable[P, R]:
            cls.events.append(Message(func, arguments, version))
            return func

        return wrapper

    @classmethod
    def request(
        cls, *arguments: Argument, version: int | None = None
    ) -> Callable[[Callable[P, R]], Callable[P, R]]:
        """Decorator for interface requests

        Adds the decorated method to the list of requests of the interface
        (client-side method).

        :param arguments: Arguments for the request
        :type arguments: :class:`~pywayland.protocol_core.Argument`
        :param version: Version of the request (optional)
        :type version: int or None
        """

        def wrapper(func: Callable[P, R]) -> Callable[P, R]:
            cls.requests.append(Message(func, arguments, version))
            return func

        return wrapper

    @classmethod
    def _gen_c(cls) -> None:
        """Creates the wl_interface C struct

        Generates the CFFI cdata for the wl_interface struct given by the
        interface.
        """
        cls.registry = WeakValueDictionary()

        name: ffi.CharCData = ffi.new("char[]", cls.name.encode())
        cls._ptr.name = name
        cls._ptr.version = cls.version

        keep_alive: list[tuple[ffi.CharCData, ffi.CharCData, ffi.WlInterface]] = []
        # Determine the number of methods to assign and assign them
        cls._ptr.method_count = len(cls.requests)
        methods_ptr: ffi.WlMessage = ffi.new("struct wl_message[]", len(cls.requests))
        cls._ptr.methods = methods_ptr

        # Iterate over the methods
        for i, message in enumerate(cls.requests):
            keep_alive.append(message.build_message_struct(methods_ptr[i]))

        cls._ptr.event_count = len(cls.events)
        events_ptr: ffi.WlMessage = ffi.new("struct wl_message[]", len(cls.events))
        cls._ptr.events = events_ptr
        # Iterate over the methods
        for i, message in enumerate(cls.events):
            keep_alive.append(message.build_message_struct(events_ptr[i]))

        weakkeydict[cls._ptr] = (name, methods_ptr, events_ptr, *tuple(keep_alive))



# This file has been autogenerated by the pywayland scanner

# Copyright © 2024 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class XdgToplevelTagManagerV1(Interface):
    """Protocol for setting toplevel tags

    In order to make some window properties like position, size, "always on
    top" or user defined rules for window behavior persistent, the compositor
    needs some way to identify windows even after the application has been
    restarted. This protocol allows clients to make this possible by setting a
    tag for toplevels.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "xdg_toplevel_tag_manager_v1"
    version = 1


class XdgToplevelTagManagerV1Resource(Resource[XdgToplevelTagManagerV1]):
    interface = XdgToplevelTagManagerV1


class XdgToplevelTagManagerV1Proxy(Proxy[XdgToplevelTagManagerV1]):
    interface = XdgToplevelTagManagerV1

    @XdgToplevelTagManagerV1.request()
    def destroy(self) -> None:
        """Destroy toplevel tag object

        Destroy this toplevel tag manager object. This request has no other
        effects.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevelTagManagerV1.request(
        Argument(ArgumentType.Object, interface=XdgToplevel),
        Argument(ArgumentType.String),
    )
    def set_toplevel_tag(self, toplevel: XdgToplevelProxy, tag: str) -> None:
        """Set tag

        Set a tag for a toplevel. The tag may be shown to the user in UI, so
        it's preferable for it to be human readable, but it must be suitable
        for configuration files and should not be translated. Suitable tags
        would for example be "main window", "settings", "e-mail composer" or
        similar.

        The tag does not need to be unique across applications, and the client
        may set the same tag for multiple windows, for example if the user has
        opened the same UI twice. How the potentially resulting conflicts are
        handled is compositor policy.

        The client should set the tag as part of the initial commit on the
        associated toplevel, but it may set it at any time afterwards as well,
        for example if the purpose of the toplevel changes.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :param tag:
            untranslated tag
        :type tag:
            `ArgumentType.String`
        """
        self._marshal(1, toplevel, tag)

    @XdgToplevelTagManagerV1.request(
        Argument(ArgumentType.Object, interface=XdgToplevel),
        Argument(ArgumentType.String),
    )
    def set_toplevel_description(self, toplevel: XdgToplevelProxy, description: str) -> None:
        """Set description

        Set a description for a toplevel. This description may be shown to the
        user in UI or read by a screen reader for accessibility purposes, and
        should be translated. It is recommended to make the description the
        translation of the tag.

        The client should set the description as part of the initial commit on
        the associated toplevel, but it may set it at any time afterwards as
        well, for example if the purpose of the toplevel changes.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :param description:
            translated description
        :type description:
            `ArgumentType.String`
        """
        self._marshal(2, toplevel, description)


class XdgToplevelTagManagerV1Global(Global[XdgToplevelTagManagerV1]):
    interface = XdgToplevelTagManagerV1


XdgToplevelTagManagerV1._gen_c()
XdgToplevelTagManagerV1.proxy_class = XdgToplevelTagManagerV1Proxy
XdgToplevelTagManagerV1.resource_class = XdgToplevelTagManagerV1Resource
XdgToplevelTagManagerV1.global_class = XdgToplevelTagManagerV1Global



from .ffi import (
    CData,
    CharCData,
    DispatcherFunc,
    EventLoopFdFunc,
    EventLoopIdleFunc,
    EventLoopSignalFunc,
    EventLoopTimerFunc,
    GlobalBindFunc,
    NotifyFunc,
    ResourceDestroyFunc,
    WlArgument,
    WlClient,
    WlDisplay,
    WlEventLoop,
    WlEventSource,
    WlGlobal,
    WlInterface,
    WlList,
    WlListener,
    WlProxy,
    WlQueue,
    WlResource,
    WlSignal,
)

###############################################################################
# wayland-version.h
###############################################################################

WAYLAND_VERSION_MAJOR: int
WAYLAND_VERSION_MINOR: int
WAYLAND_VERSION_MICRO: int

###############################################################################
# wayland-util.h
###############################################################################

# wl_fixed_t methods
def wl_fixed_to_double(f: int) -> float: ...
def wl_fixed_from_double(d: float) -> int: ...
def wl_fixed_from_int(i: int) -> int: ...

# wl_list methods
def wl_list_init(list: WlList) -> None: ...
def wl_list_insert(list: WlList, elm: WlList) -> None: ...
def wl_list_remove(list: WlList) -> None: ...

###############################################################################
# wayland-client-core.h
###############################################################################

# wl_event_queue methods
def wl_event_queue_destroy(queue: WlQueue) -> None: ...

# wl_proxy methods
def wl_proxy_marshal_array(
    proxy: WlProxy, opcode: int, args_ptr: WlArgument
) -> None: ...
def wl_proxy_marshal_array_constructor(
    proxy: WlProxy, opcode: int, args_ptr: WlArgument, interface_ptr: WlInterface
) -> WlProxy: ...
def wl_proxy_destroy(proxy: WlProxy) -> None: ...
def wl_proxy_add_dispatcher(
    proxy: WlProxy, dispatcher_func: DispatcherFunc, dispatcher_data: CData, data: CData
) -> int: ...

# wl_display methods
def wl_display_connect(name: bytes | CharCData) -> WlDisplay: ...
def wl_display_connect_to_fd(fd: int) -> WlDisplay: ...
def wl_display_disconnect(display: WlDisplay) -> None: ...
def wl_display_get_fd(display: WlDisplay) -> int: ...
def wl_display_dispatch(display: WlDisplay) -> int: ...
def wl_display_dispatch_queue(display: WlDisplay, queue: WlQueue) -> int: ...
def wl_display_dispatch_queue_pending(display: WlDisplay, queue: WlQueue) -> int: ...
def wl_display_dispatch_pending(display: WlDisplay) -> int: ...
def wl_display_get_error(display: WlDisplay) -> int: ...
def wl_display_flush(display: WlDisplay) -> int: ...
def wl_display_roundtrip_queue(display: WlDisplay, queue: WlQueue) -> int: ...
def wl_display_roundtrip(display: WlDisplay) -> int: ...
def wl_display_create_queue(display: WlDisplay) -> WlQueue: ...
def wl_display_prepare_read_queue(display: WlDisplay, queue: WlQueue) -> int: ...
def wl_display_prepare_read(display: WlDisplay) -> int: ...
def wl_display_read_events(display: WlDisplay) -> int: ...

###############################################################################
# wayland-server-core.h
###############################################################################

# event kind enum
WL_EVENT_READABLE: int
WL_EVENT_WRITABLE: int
WL_EVENT_HANGUP: int
WL_EVENT_ERROR: int

# wl_signal methods
def wl_signal_init(ptr: WlSignal) -> None: ...
def wl_signal_add(ptr: WlSignal, listener: WlListener) -> None: ...
def wl_signal_emit(ptr: WlSignal, data: CData) -> None: ...

# wl_event_loop methods
def wl_event_loop_create() -> WlEventLoop: ...
def wl_event_loop_destroy(event_loop: WlEventLoop) -> None: ...
def wl_event_loop_add_fd(
    ptr: WlEventLoop,
    fd: int,
    mask: int,
    event_loop_fd_func: EventLoopFdFunc,
    data: CData,
) -> WlEventSource: ...
def wl_event_loop_add_timer(
    ptr: WlEventLoop, event_loop_timer_func: EventLoopTimerFunc, data: CData
) -> WlEventSource: ...
def wl_event_loop_add_signal(
    ptr: WlEventLoop,
    signal: int,
    event_loop_signal_func: EventLoopSignalFunc,
    data: CData,
) -> WlEventSource: ...
def wl_event_loop_dispatch(ptr: WlEventLoop, timeout: int) -> int: ...
def wl_event_loop_dispatch_idle(ptr: WlEventLoop) -> None: ...
def wl_event_loop_add_idle(
    ptr: WlEventLoop, event_loop_idle_func: EventLoopIdleFunc, data: CData
) -> WlEventSource: ...
def wl_event_loop_add_destroy_listener(
    ptr: WlEventLoop, listener: WlListener
) -> None: ...

# wl_event_source methods
def wl_event_source_timer_update(ptr: WlEventSource, ms_delay: int) -> int: ...
def wl_event_source_remove(ptr: WlEventSource) -> int: ...
def wl_event_source_check(ptr: WlEventSource) -> None: ...

# wl_display methods
def wl_display_create() -> WlDisplay: ...
def wl_display_destroy(display: WlDisplay) -> None: ...
def wl_display_get_event_loop(display: WlDisplay) -> WlEventLoop: ...
def wl_display_add_socket(display: WlDisplay, name: bytes | CharCData) -> int: ...
def wl_display_add_socket_auto(display: WlDisplay) -> CharCData: ...
def wl_display_terminate(display: WlDisplay) -> None: ...
def wl_display_run(display: WlDisplay) -> None: ...
def wl_display_flush_clients(display: WlDisplay) -> None: ...
def wl_display_destroy_clients(display: WlDisplay) -> None: ...
def wl_display_get_serial(display: WlDisplay) -> int: ...
def wl_display_next_serial(display: WlDisplay) -> int: ...
def wl_display_init_shm(display: WlDisplay) -> int: ...
def wl_display_add_shm_format(display: WlDisplay, format: int) -> CData: ...

# wl_global methods
def wl_global_create(
    display: WlDisplay,
    interface: WlInterface,
    version: int,
    data: CData,
    global_bind_func: GlobalBindFunc,
) -> WlGlobal: ...
def wl_global_destroy(data: WlGlobal) -> None: ...

# wl_client methods
def wl_client_create(display: WlDisplay, fd: int) -> WlClient: ...
def wl_client_destroy(client: WlClient) -> None: ...
def wl_client_flush(client: WlClient) -> None: ...
def wl_client_get_credentials(
    client: WlClient, pid: CData, uid: CData, gid: CData
) -> None: ...
def wl_client_add_destroy_listener(client: WlClient, listener: WlListener) -> None: ...
def wl_client_get_object(client: WlClient, id: int) -> WlResource: ...

# wl_resource methods
def wl_resource_post_event_array(
    resource: WlResource, opcode: int, args: WlArgument
) -> None: ...
def wl_resource_post_error(
    resource: WlResource, code: int, msg: bytes | CharCData
) -> None: ...
def wl_resource_create(
    client: WlClient, interface: WlInterface, version: int, id: int
) -> WlResource: ...
def wl_resource_set_dispatcher(
    resource: WlResource,
    dispatcher_func: DispatcherFunc,
    implementation: CData,
    data: CData,
    resource_destroy_func: ResourceDestroyFunc,
) -> None: ...
def wl_resource_destroy(resource: WlResource) -> None: ...
def wl_resource_get_id(resource: WlResource) -> int: ...
def wl_resource_get_client(resource: WlResource) -> WlClient: ...
def wl_resource_get_user_data(resource: WlResource) -> CData: ...
def wl_resource_add_destroy_listener(
    resource: WlResource, listener: WlListener
) -> None: ...

# anonymous file methods
def os_create_anonymous_file(size: int) -> int: ...

# cffi callback functions
dispatcher_func: DispatcherFunc
resource_destroy_func: ResourceDestroyFunc
event_loop_fd_func: EventLoopFdFunc
event_loop_signal_func: EventLoopSignalFunc
event_loop_timer_func: EventLoopTimerFunc
event_loop_idle_func: EventLoopIdleFunc
global_bind_func: GlobalBindFunc
notify_func: NotifyFunc



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Ilia Bozhinov
# Copyright © 2020 Isaac Freund
# Copyright © 2022 wb9688
# Copyright © 2023 i509VCB
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ExtForeignToplevelHandleV1(Interface):
    """A mapped toplevel

    A :class:`ExtForeignToplevelHandleV1` object represents a mapped toplevel
    window. A single app may have multiple mapped toplevels.
    """

    name = "ext_foreign_toplevel_handle_v1"
    version = 1


class ExtForeignToplevelListV1(Interface):
    """List toplevels

    A toplevel is defined as a surface with a role similar to
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`. XWayland surfaces may
    be treated like toplevels in this protocol.

    After a client binds the :class:`ExtForeignToplevelListV1`, each mapped
    toplevel window will be sent using the
    :func:`ExtForeignToplevelListV1.toplevel()` event.

    Clients which only care about the current state can perform a roundtrip
    after binding this global.

    For each instance of :class:`ExtForeignToplevelListV1`, the compositor must
    create a new
    :class:`~pywayland.protocol.ext_foreign_toplevel_list_v1.ExtForeignToplevelHandleV1`
    object for each mapped toplevel.

    If a compositor implementation sends the
    :func:`ExtForeignToplevelListV1.finished()` event after the global is
    bound, the compositor must not send any
    :func:`ExtForeignToplevelListV1.toplevel()` events.
    """

    name = "ext_foreign_toplevel_list_v1"
    version = 1


class ExtForeignToplevelHandleV1Resource(Resource[ExtForeignToplevelHandleV1]):
    interface = ExtForeignToplevelHandleV1

    @ExtForeignToplevelHandleV1.event()
    def closed(self) -> None:
        """The toplevel has been closed

        The server will emit no further events on the
        :class:`ExtForeignToplevelHandleV1` after this event. Any requests
        received aside from the destroy request must be ignored. Upon receiving
        this event, the client should destroy the handle.

        Other protocols which extend the :class:`ExtForeignToplevelHandleV1`
        interface must also ignore requests other than destructors.
        """
        self._post_event(0)

    @ExtForeignToplevelHandleV1.event()
    def done(self) -> None:
        """All information about the toplevel has been sent

        This event is sent after all changes in the toplevel state have been
        sent.

        This allows changes to the :class:`ExtForeignToplevelHandleV1`
        properties to be atomically applied. Other protocols which extend the
        :class:`ExtForeignToplevelHandleV1` interface may use this event to
        also atomically apply any pending state.

        This event must not be sent after the
        :func:`ExtForeignToplevelHandleV1.closed()` event.
        """
        self._post_event(1)

    @ExtForeignToplevelHandleV1.event(
        Argument(ArgumentType.String),
    )
    def title(self, title: str) -> None:
        """Title change

        The title of the toplevel has changed.

        The configured state must not be applied immediately. See
        :func:`ExtForeignToplevelHandleV1.done()` for details.

        :param title:
        :type title:
            `ArgumentType.String`
        """
        self._post_event(2, title)

    @ExtForeignToplevelHandleV1.event(
        Argument(ArgumentType.String),
    )
    def app_id(self, app_id: str) -> None:
        """App_id change

        The app id of the toplevel has changed.

        The configured state must not be applied immediately. See
        :func:`ExtForeignToplevelHandleV1.done()` for details.

        :param app_id:
        :type app_id:
            `ArgumentType.String`
        """
        self._post_event(3, app_id)

    @ExtForeignToplevelHandleV1.event(
        Argument(ArgumentType.String),
    )
    def identifier(self, identifier: str) -> None:
        """A stable identifier for a toplevel

        This identifier is used to check if two or more toplevel handles belong
        to the same toplevel.

        The identifier is useful for command line tools or privileged clients
        which may need to reference an exact toplevel across processes or
        instances of the
        :class:`~pywayland.protocol.ext_foreign_toplevel_list_v1.ExtForeignToplevelListV1`
        global.

        The compositor must only send this event when the handle is created.

        The identifier must be unique per toplevel and its handles. Two
        different toplevels must not have the same identifier. The identifier
        is only valid as long as the toplevel is mapped. If the toplevel is
        unmapped the identifier must not be reused. An identifier must not be
        reused by the compositor to ensure there are no races when sharing
        identifiers between processes.

        An identifier is a string that contains up to 32 printable ASCII bytes.
        An identifier must not be an empty string. It is recommended that a
        compositor includes an opaque generation value in identifiers. How the
        generation value is used when generating the identifier is
        implementation dependent.

        :param identifier:
        :type identifier:
            `ArgumentType.String`
        """
        self._post_event(4, identifier)


class ExtForeignToplevelListV1Resource(Resource[ExtForeignToplevelListV1]):
    interface = ExtForeignToplevelListV1

    @ExtForeignToplevelListV1.event(
        Argument(ArgumentType.NewId, interface=ExtForeignToplevelHandleV1),
    )
    def toplevel(self, toplevel: ExtForeignToplevelHandleV1) -> None:
        """A toplevel has been created

        This event is emitted whenever a new toplevel window is created. It is
        emitted for all toplevels, regardless of the app that has created them.

        All initial properties of the toplevel (identifier, title, app_id) will
        be sent immediately after this event using the corresponding events for
        :class:`~pywayland.protocol.ext_foreign_toplevel_list_v1.ExtForeignToplevelHandleV1`.
        The compositor will use the :func:`ExtForeignToplevelHandleV1.done()
        <pywayland.protocol.ext_foreign_toplevel_list_v1.ExtForeignToplevelHandleV1.done>`
        event to indicate when all data has been sent.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.ext_foreign_toplevel_list_v1.ExtForeignToplevelHandleV1`
        """
        self._post_event(0, toplevel)

    @ExtForeignToplevelListV1.event()
    def finished(self) -> None:
        """The compositor has finished with the toplevel manager

        This event indicates that the compositor is done sending events to this
        object. The client should destroy the object. See
        :func:`ExtForeignToplevelListV1.destroy()` for more information.

        The compositor must not send any more toplevel events after this event.
        """
        self._post_event(1)


class ExtForeignToplevelHandleV1Proxy(Proxy[ExtForeignToplevelHandleV1]):
    interface = ExtForeignToplevelHandleV1

    @ExtForeignToplevelHandleV1.request()
    def destroy(self) -> None:
        """Destroy the :class:`ExtForeignToplevelHandleV1` object

        This request should be used when the client will no longer use the
        handle or after the closed event has been received to allow destruction
        of the object.

        When a handle is destroyed, a new handle may not be created by the
        server until the toplevel is unmapped and then remapped. Destroying a
        toplevel handle is not recommended unless the client is cleaning up
        child objects before destroying the
        :class:`~pywayland.protocol.ext_foreign_toplevel_list_v1.ExtForeignToplevelListV1`
        object, the toplevel was closed or the toplevel handle will not be used
        in the future.

        Other protocols which extend the :class:`ExtForeignToplevelHandleV1`
        interface should require destructors for extension interfaces be called
        before allowing the toplevel handle to be destroyed.
        """
        self._marshal(0)
        self._destroy()


class ExtForeignToplevelListV1Proxy(Proxy[ExtForeignToplevelListV1]):
    interface = ExtForeignToplevelListV1

    @ExtForeignToplevelListV1.request()
    def stop(self) -> None:
        """Stop sending events

        This request indicates that the client no longer wishes to receive
        events for new toplevels.

        The Wayland protocol is asynchronous, meaning the compositor may send
        further toplevel events until the stop request is processed. The client
        should wait for a :func:`ExtForeignToplevelListV1.finished()` event
        before destroying this object.
        """
        self._marshal(0)

    @ExtForeignToplevelListV1.request()
    def destroy(self) -> None:
        """Destroy the :class:`ExtForeignToplevelListV1` object

        This request should be called either when the client will no longer use
        the :class:`ExtForeignToplevelListV1` or after the finished event has
        been received to allow destruction of the object.

        If a client wishes to destroy this object it should send a
        :func:`ExtForeignToplevelListV1.stop()` request and wait for a
        :func:`ExtForeignToplevelListV1.finished()` event, then destroy the
        handles and then this object.
        """
        self._marshal(1)
        self._destroy()


class ExtForeignToplevelHandleV1Global(Global[ExtForeignToplevelHandleV1]):
    interface = ExtForeignToplevelHandleV1


class ExtForeignToplevelListV1Global(Global[ExtForeignToplevelListV1]):
    interface = ExtForeignToplevelListV1


ExtForeignToplevelHandleV1._gen_c()
ExtForeignToplevelHandleV1.proxy_class = ExtForeignToplevelHandleV1Proxy
ExtForeignToplevelHandleV1.resource_class = ExtForeignToplevelHandleV1Resource
ExtForeignToplevelHandleV1.global_class = ExtForeignToplevelHandleV1Global


ExtForeignToplevelListV1._gen_c()
ExtForeignToplevelListV1.proxy_class = ExtForeignToplevelListV1Proxy
ExtForeignToplevelListV1.resource_class = ExtForeignToplevelListV1Resource
ExtForeignToplevelListV1.global_class = ExtForeignToplevelListV1Global



from collections.abc import Callable
from typing import Any, ParamSpec, Self, TypeVar, overload

class CData:
    def __getitem__(self, idx: int) -> Self: ...
    def __setitem__(self, idx: int, elem: Self) -> None: ...

# built-in cdata types
class CharCData(CData): ...

# extern functions
class DispatcherFunc: ...
class ResourceDestroyFunc: ...
class EventLoopFdFunc: ...
class EventLoopSignalFunc: ...
class EventLoopTimerFunc: ...
class EventLoopIdleFunc: ...
class GlobalBindFunc: ...
class NotifyFunc: ...

class WlArgument(CData):
    i: int
    u: int
    f: int
    s: CharCData
    o: WlObject
    n: int
    a: WlArray
    h: int

class WlArray(CData):
    size: int
    alloc: int
    data: CData

class WlClient(CData):
    pass

class WlDisplay(CData):
    pass

class WlEventLoop(CData):
    pass

class WlEventSource(CData):
    pass

class WlGlobal(CData):
    pass

class WlInterface(CData):
    name: CharCData
    version: int
    method_count: int
    methods: WlMessage
    event_count: int
    events: WlMessage

class WlList(CData):
    @property
    def prev(self) -> WlList: ...
    @property
    def next(self) -> WlList: ...

class WlListener(CData):
    link: WlList
    notify: NotifyFunc

class WlListenerContainer(CData):
    handle: CData
    destroy_listener: WlListener

class WlMessage(CData):
    name: CharCData
    signature: CharCData
    types: WlInterface

class WlObject(CData):
    pass

class WlProxy(CData):
    pass

class WlQueue(CData):
    pass

class WlResource(CData):
    pass

class WlSignal(CData):
    listener_list: WlList

# special types
P = ParamSpec("P")
R = TypeVar("R")
_CDataT = TypeVar("_CDataT", bound=CData)
_CDataO = TypeVar("_CDataO", bound=CData)

# Any type of CData
NULL: Any

def new(cdecl: str, init: Any = None) -> _CDataT: ...  # type: ignore [type-var]
@overload
def gc(
    cdata: _CDataT, destructor: Callable[[_CDataT], Any], size: int = 0
) -> _CDataT: ...
@overload
def gc(cdata: _CDataT, destructor: None, size: int = 0) -> None: ...
def buffer(cdata: _CDataT, size: int = -1) -> bytearray: ...
def string(cdata: CharCData, maxlen: int = -1) -> bytes: ...
def release(x: _CDataT) -> None: ...
def def_extern() -> Callable[[Callable[P, R]], Callable[P, R]]: ...
def new_handle(x: Any) -> _CDataT: ...  # type: ignore [type-var]
def from_handle(x: _CDataT) -> Any: ...
def cast(cdecl: str, source: _CDataT) -> _CDataO: ...  # type: ignore [type-var]
def addressof(cdata: _CDataT, *fields_or_indexes: str | int) -> _CDataT: ...
def offsetof(cdecl: str, *fields_or_indexes: str | int) -> int: ...



# This file has been autogenerated by the pywayland scanner

# Copyright © 2023 Valve Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpFifoManagerV1(Interface):
    """Protocol for fifo constraints

    When a Wayland compositor considers applying a content update, it must
    ensure all the update's readiness constraints (fences, etc) are met.

    This protocol provides a way to use the completion of a display refresh
    cycle as an additional readiness constraint.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_fifo_manager_v1"
    version = 1

    class error(enum.IntEnum):
        already_exists = 0


class WpFifoV1(Interface):
    """Fifo interface

    A fifo object for a surface that may be used to add display refresh
    constraints to content updates.
    """

    name = "wp_fifo_v1"
    version = 1

    class error(enum.IntEnum):
        surface_destroyed = 0


class WpFifoManagerV1Resource(Resource[WpFifoManagerV1]):
    interface = WpFifoManagerV1


class WpFifoV1Resource(Resource[WpFifoV1]):
    interface = WpFifoV1


class WpFifoManagerV1Proxy(Proxy[WpFifoManagerV1]):
    interface = WpFifoManagerV1

    @WpFifoManagerV1.request()
    def destroy(self) -> None:
        """Unbind from the manager interface

        Informs the server that the client will no longer be using this
        protocol object. Existing objects created by this object are not
        affected.
        """
        self._marshal(0)
        self._destroy()

    @WpFifoManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpFifoV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_fifo(self, surface: WlSurfaceProxy) -> WpFifoV1Proxy:
        """Request fifo interface for surface

        Establish a fifo object for a surface that may be used to add display
        refresh constraints to content updates.

        Only one such object may exist for a surface and attempting to create
        more than one will result in an already_exists protocol error. If a
        surface is acted on by multiple software components, general best
        practice is that only the component performing
        :func:`WlSurface.attach()
        <pywayland.protocol.wayland.WlSurface.attach>` operations should use
        this protocol.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.fifo_v1.WpFifoV1`
        """
        id = self._marshal_constructor(1, WpFifoV1, surface)
        return id


class WpFifoV1Proxy(Proxy[WpFifoV1]):
    interface = WpFifoV1

    @WpFifoV1.request()
    def set_barrier(self) -> None:
        """Sets the start point for a fifo constraint

        When the content update containing the "set_barrier" is applied, it
        sets a "fifo_barrier" condition on the surface associated with the fifo
        object. The condition is cleared immediately after the following
        latching deadline for non-tearing presentation.

        The compositor may clear the condition early if it must do so to ensure
        client forward progress assumptions.

        To wait for this condition to clear, use the "wait_barrier" request.

        "set_barrier" is double-buffered state, see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        Requesting set_barrier after the fifo object's surface is destroyed
        will generate a "surface_destroyed" error.
        """
        self._marshal(0)

    @WpFifoV1.request()
    def wait_barrier(self) -> None:
        """Adds a fifo constraint to a content update

        Indicate that this content update is not ready while a "fifo_barrier"
        condition is present on the surface.

        This means that when the content update containing "set_barrier" was
        made active at a latching deadline, it will be active for at least one
        refresh cycle. A content update which is allowed to tear might become
        active after a latching deadline if no content update became active at
        the deadline.

        The constraint must be ignored if the surface is a subsurface in
        synchronized mode. If the surface is not being updated by the
        compositor (off-screen, occluded) the compositor may ignore the
        constraint. Clients must use an additional mechanism such as frame
        callbacks or timestamps to ensure throttling occurs under all
        conditions.

        "wait_barrier" is double-buffered state, see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        Requesting "wait_barrier" after the fifo object's surface is destroyed
        will generate a "surface_destroyed" error.
        """
        self._marshal(1)

    @WpFifoV1.request()
    def destroy(self) -> None:
        """Destroy the fifo interface

        Informs the server that the client will no longer be using this
        protocol object.

        Surface state changes previously made by this protocol are unaffected
        by this object's destruction.
        """
        self._marshal(2)
        self._destroy()


class WpFifoManagerV1Global(Global[WpFifoManagerV1]):
    interface = WpFifoManagerV1


class WpFifoV1Global(Global[WpFifoV1]):
    interface = WpFifoV1


WpFifoManagerV1._gen_c()
WpFifoManagerV1.proxy_class = WpFifoManagerV1Proxy
WpFifoManagerV1.resource_class = WpFifoManagerV1Resource
WpFifoManagerV1.global_class = WpFifoManagerV1Global


WpFifoV1._gen_c()
WpFifoV1.proxy_class = WpFifoV1Proxy
WpFifoV1.resource_class = WpFifoV1Resource
WpFifoV1.global_class = WpFifoV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014, 2015 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlBuffer
from .wayland import WlBufferProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZwpLinuxBufferParamsV1(Interface):
    """Parameters for creating a dmabuf-based :class:`~pywayland.protocol.wayland.WlBuffer`

    This temporary object is a collection of dmabufs and other parameters that
    together form a single logical buffer. The temporary object may eventually
    create one :class:`~pywayland.protocol.wayland.WlBuffer` unless cancelled
    by destroying it before requesting 'create'.

    Single-planar formats only require one dmabuf, however multi-planar formats
    may require more than one dmabuf. For all formats, an 'add' request must be
    called once per plane (even if the underlying dmabuf fd is identical).

    You must use consecutive plane indices ('plane_idx' argument for 'add')
    from zero to the number of planes used by the drm_fourcc format code. All
    planes required by the format must be given exactly once, but can be given
    in any order. Each plane index can only be set once; subsequent calls with
    a plane index which has already been set will result in a plane_set error
    being generated.
    """

    name = "zwp_linux_buffer_params_v1"
    version = 6

    class error(enum.IntEnum):
        already_used = 0
        plane_idx = 1
        plane_set = 2
        incomplete = 3
        invalid_format = 4
        invalid_dimensions = 5
        out_of_bounds = 6
        invalid_wl_buffer = 7
        invalid_dev_t_size = 8

    class flags(enum.IntFlag):
        y_invert = 1
        interlaced = 2
        bottom_first = 4


class ZwpLinuxDmabufFeedbackV1(Interface):
    """Dmabuf feedback

    This object advertises dmabuf parameters feedback. This includes the
    preferred devices and the supported formats/modifiers.

    The parameters are sent once when this object is created and whenever they
    change. The done event is always sent once after all parameters have been
    sent. When a single parameter changes, all parameters are re-sent by the
    compositor.

    Compositors can re-send the parameters when the current client buffer
    allocations are sub-optimal. Compositors should not re-send the parameters
    if re-allocating the buffers would not result in a more optimal
    configuration. In particular, compositors should avoid sending the exact
    same parameters multiple times in a row.

    The tranche_target_device and tranche_formats events are grouped by
    tranches of preference. For each tranche, a tranche_target_device, one
    tranche_flags and one or more tranche_formats events are sent, followed by
    a tranche_done event finishing the list. The tranches are sent in
    descending order of preference. All formats and modifiers in the same
    tranche have the same preference.

    To send parameters, the compositor sends one main_device event (unless the
    client bound version 6 or above), tranches (each consisting of one
    tranche_target_device event, one tranche_flags event, tranche_formats
    events and then a tranche_done event), then one done event.

    With version 6 and above, the compositor must always advertise at least one
    tranche with the sampling flag set.
    """

    name = "zwp_linux_dmabuf_feedback_v1"
    version = 6

    class tranche_flags(enum.IntFlag):
        scanout = 1
        sampling = 2


class ZwpLinuxDmabufV1(Interface):
    """Factory for creating dmabuf-based wl_buffers

    This interface offers ways to create generic dmabuf-based wl_buffers.

    For more information about dmabuf, see:
    https://www.kernel.org/doc/html/next/userspace-api/dma-buf-alloc-
    exchange.html

    Clients can use the get_surface_feedback request to get dmabuf feedback for
    a particular surface. If the client wants to retrieve feedback not tied to
    a surface, they can use the get_default_feedback request.

    The following are required from clients:

    - Clients must ensure that either all data in the dma-buf is coherent for
      all subsequent read access or that coherency is correctly handled by the
      underlying kernel-side dma-buf implementation.

    - Don't make any more attachments after sending the buffer to the
      compositor. Making more attachments later increases the risk of the
      compositor not being able to use (re-import) an existing dmabuf-based
      :class:`~pywayland.protocol.wayland.WlBuffer`.

    The underlying graphics stack must ensure the following:

    - The dmabuf file descriptors relayed to the server will stay valid for the
      whole lifetime of the :class:`~pywayland.protocol.wayland.WlBuffer`. This
      means the server may at any time use those fds to import the dmabuf into
      any kernel sub-system that might accept it.

    However, when the underlying graphics stack fails to deliver the promise,
    because of e.g. a device hot-unplug which raises internal errors, after the
    :class:`~pywayland.protocol.wayland.WlBuffer` has been successfully created
    the compositor must not raise protocol errors to the client when dmabuf
    import later fails.

    To create a :class:`~pywayland.protocol.wayland.WlBuffer` from one or more
    dmabufs, a client creates a
    :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1` object
    with a :func:`ZwpLinuxDmabufV1.create_params()` request. All planes
    required by the intended format are added with the 'add' request. Finally,
    a 'create' or 'create_immed' request is issued, which has the following
    outcome depending on the import success.

    The 'create' request, - on success, triggers a 'created' event which
    provides the final   :class:`~pywayland.protocol.wayland.WlBuffer` to the
    client. - on failure, triggers a 'failed' event to convey that the server
    cannot use the dmabufs received from the client.

    For the 'create_immed' request, - on success, the server immediately
    imports the added dmabufs to   create a
    :class:`~pywayland.protocol.wayland.WlBuffer`. No event is sent from the
    server in this case. - on failure, the server can choose to either:   -
    terminate the client by raising a fatal error.   - mark the
    :class:`~pywayland.protocol.wayland.WlBuffer` as failed, and send a
    'failed' event to the     client. If the client uses a failed
    :class:`~pywayland.protocol.wayland.WlBuffer` as an argument to any
    request, the behaviour is compositor implementation-defined.

    For all DRM formats and unless specified in another protocol extension,
    pre-multiplied alpha is used for pixel values.

    Unless specified otherwise in another protocol extension, implicit
    synchronization is used. In other words, compositors and clients must wait
    and signal fences implicitly passed via the DMA-BUF's reservation
    mechanism.
    """

    name = "zwp_linux_dmabuf_v1"
    version = 6


class ZwpLinuxBufferParamsV1Resource(Resource[ZwpLinuxBufferParamsV1]):
    interface = ZwpLinuxBufferParamsV1

    @ZwpLinuxBufferParamsV1.event(
        Argument(ArgumentType.NewId, interface=WlBuffer),
    )
    def created(self, buffer: WlBuffer) -> None:
        """Buffer creation succeeded

        This event indicates that the attempted buffer creation was successful.
        It provides the new :class:`~pywayland.protocol.wayland.WlBuffer`
        referencing the dmabuf(s).

        Upon receiving this event, the client should destroy the
        :class:`ZwpLinuxBufferParamsV1` object.

        :param buffer:
            id for the the newly created
            :class:`~pywayland.protocol.wayland.WlBuffer`
        :type buffer:
            :class:`~pywayland.protocol.wayland.WlBuffer`
        """
        self._post_event(0, buffer)

    @ZwpLinuxBufferParamsV1.event()
    def failed(self) -> None:
        """Buffer creation failed

        This event indicates that the attempted buffer creation has failed. It
        usually means that one of the dmabuf constraints has not been
        fulfilled.

        Upon receiving this event, the client should destroy the
        :class:`ZwpLinuxBufferParamsV1` object.
        """
        self._post_event(1)


class ZwpLinuxDmabufFeedbackV1Resource(Resource[ZwpLinuxDmabufFeedbackV1]):
    interface = ZwpLinuxDmabufFeedbackV1

    @ZwpLinuxDmabufFeedbackV1.event()
    def done(self) -> None:
        """All feedback has been sent

        This event is sent after all parameters of a
        :class:`ZwpLinuxDmabufFeedbackV1` object have been sent.

        This allows changes to the :class:`ZwpLinuxDmabufFeedbackV1` parameters
        to be seen as atomic, even if they happen via multiple events.
        """
        self._post_event(0)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
    )
    def format_table(self, fd: int, size: int) -> None:
        """Format and modifier table

        This event provides a file descriptor which can be memory-mapped to
        access the format and modifier table.

        The table contains a tightly packed array of consecutive format +
        modifier pairs. Each pair is 16 bytes wide. It contains a format as a
        32-bit unsigned integer, followed by 4 bytes of unused padding, and a
        modifier as a 64-bit unsigned integer. The native endianness is used.

        The client must map the file descriptor in read-only private mode.

        Compositors are not allowed to mutate the table file contents once this
        event has been sent. Instead, compositors must create a new, separate
        table file and re-send feedback parameters. Compositors are allowed to
        store duplicate format + modifier pairs in the table.

        :param fd:
            table file descriptor
        :type fd:
            `ArgumentType.FileDescriptor`
        :param size:
            table size, in bytes
        :type size:
            `ArgumentType.Uint`
        """
        self._post_event(1, fd, size)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Array),
    )
    def main_device(self, device: list) -> None:
        """Preferred main device

        This event advertises the main device that the server prefers to use
        when direct scan-out to the target device isn't possible. The
        advertised main device may be different for each
        :class:`ZwpLinuxDmabufFeedbackV1` object, and may change over time.

        There is exactly one main device. The compositor must send at least one
        preference tranche with tranche_target_device equal to main_device.

        Clients need to create buffers that the main device can import and read
        from, otherwise creating the dmabuf
        :class:`~pywayland.protocol.wayland.WlBuffer` will fail (see the
        :func:`ZwpLinuxBufferParamsV1.create()
        <pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1.create>` and
        create_immed requests for details). The main device will also likely be
        kept active by the compositor, so clients can use it instead of waking
        up another device for power savings.

        In general the device is a DRM node. The DRM node type (primary vs.
        render) is unspecified. Clients must not rely on the compositor sending
        a particular node type. Clients cannot check two devices for equality
        by comparing the dev_t value.

        If explicit modifiers are not supported and the client performs buffer
        allocations on a different device than the main device, then the client
        must force the buffer to have a linear layout.

        With version 6 and above, this event is no longer sent. Clients should
        use a device with the sampling flag in the tranches instead.

        :param device:
            device dev_t value
        :type device:
            `ArgumentType.Array`
        """
        self._post_event(2, device)

    @ZwpLinuxDmabufFeedbackV1.event()
    def tranche_done(self) -> None:
        """A preference tranche has been sent

        This event splits tranche_target_device and tranche_formats events into
        preference tranches. It is sent after a set of tranche_target_device
        and tranche_formats events; it represents the end of a tranche. The
        next tranche will have a lower preference.
        """
        self._post_event(3)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Array),
    )
    def tranche_target_device(self, device: list) -> None:
        """Target device

        This event advertises the target device that the server prefers to use
        for a buffer created given this tranche. The advertised target device
        may be different for each preference tranche, and may change over time.

        There is exactly one target device per tranche.

        The target device may be a scan-out device, for example if the
        compositor prefers to directly scan-out a buffer created given this
        tranche. The target device may be a rendering device, for example if
        the compositor prefers to texture from said buffer.

        The client can use this hint to allocate the buffer in a way that makes
        it accessible from the target device, ideally directly. The buffer must
        still be accessible from a device with the sampling flag, either
        through direct import or a potentially more expensive fallback path. If
        the buffer can't be directly imported for sampling, then clients must
        be prepared for the compositor changing the tranche priority or making
        :class:`~pywayland.protocol.wayland.WlBuffer` creation fail (see the
        :func:`ZwpLinuxBufferParamsV1.create()
        <pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1.create>` and
        create_immed requests for details).

        If the device is a DRM node, the DRM node type (primary vs. render) is
        unspecified. Clients must not rely on the compositor sending a
        particular node type. Clients cannot check two devices for equality by
        comparing the dev_t value.

        This event is tied to a preference tranche, see the tranche_done event.

        :param device:
            device dev_t value
        :type device:
            `ArgumentType.Array`
        """
        self._post_event(4, device)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Array),
    )
    def tranche_formats(self, indices: list) -> None:
        """Supported buffer format modifiers

        This event advertises the format + modifier combinations that the
        compositor supports.

        It carries an array of indices, each referring to a format + modifier
        pair in the last received format table (see the format_table event).
        Each index is a 16-bit unsigned integer in native endianness.

        For legacy support, DRM_FORMAT_MOD_INVALID is an allowed modifier. It
        indicates that the server can support the format with an implicit
        modifier. When a buffer has DRM_FORMAT_MOD_INVALID as its modifier, it
        is as if no explicit modifier is specified. The effective modifier will
        be derived from the dmabuf.

        A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for
        a given format supports both explicit modifiers and implicit modifiers.

        Compositors must not send duplicate format + modifier pairs within the
        same tranche or across two different tranches with the same target
        device and flags.

        This event is tied to a preference tranche, see the tranche_done event.

        For the definition of the format and modifier codes, see the
        :func:`ZwpLinuxBufferParamsV1.create()
        <pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1.create>`
        request.

        :param indices:
            array of 16-bit indexes
        :type indices:
            `ArgumentType.Array`
        """
        self._post_event(5, indices)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Uint),
    )
    def tranche_flags(self, flags: int) -> None:
        """Tranche flags

        This event sets tranche-specific flags. This event is tied to a
        preference tranche, see the tranche_done event. With version 6 and
        above, the compositor must set at least one flag in each tranche.

        :param flags:
            tranche flags
        :type flags:
            `ArgumentType.Uint`
        """
        self._post_event(6, flags)


class ZwpLinuxDmabufV1Resource(Resource[ZwpLinuxDmabufV1]):
    interface = ZwpLinuxDmabufV1

    @ZwpLinuxDmabufV1.event(
        Argument(ArgumentType.Uint),
    )
    def format(self, format: int) -> None:
        """Supported buffer format

        This event advertises one buffer format that the server supports. All
        the supported formats are advertised once when the client binds to this
        interface. A roundtrip after binding guarantees that the client has
        received all supported formats.

        For the definition of the format codes, see the
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`::create
        request.

        Starting version 4, the format event is deprecated and must not be sent
        by compositors. Instead, use get_default_feedback or
        get_surface_feedback.

        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        """
        self._post_event(0, format)

    @ZwpLinuxDmabufV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def modifier(self, format: int, modifier_hi: int, modifier_lo: int) -> None:
        """Supported buffer format modifier

        This event advertises the formats that the server supports, along with
        the modifiers supported for each format. All the supported modifiers
        for all the supported formats are advertised once when the client binds
        to this interface. A roundtrip after binding guarantees that the client
        has received all supported format-modifier pairs.

        For legacy support, DRM_FORMAT_MOD_INVALID (that is, modifier_hi ==
        0x00ffffff and modifier_lo == 0xffffffff) is allowed in this event. It
        indicates that the server can support the format with an implicit
        modifier. When a plane has DRM_FORMAT_MOD_INVALID as its modifier, it
        is as if no explicit modifier is specified. The effective modifier will
        be derived from the dmabuf.

        A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for
        a given format supports both explicit modifiers and implicit modifiers.

        For the definition of the format and modifier codes, see the
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`::create
        and
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`::add
        requests.

        Starting version 4, the modifier event is deprecated and must not be
        sent by compositors. Instead, use get_default_feedback or
        get_surface_feedback.

        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param modifier_hi:
            high 32 bits of layout modifier
        :type modifier_hi:
            `ArgumentType.Uint`
        :param modifier_lo:
            low 32 bits of layout modifier
        :type modifier_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, format, modifier_hi, modifier_lo)


class ZwpLinuxBufferParamsV1Proxy(Proxy[ZwpLinuxBufferParamsV1]):
    interface = ZwpLinuxBufferParamsV1

    @ZwpLinuxBufferParamsV1.request()
    def destroy(self) -> None:
        """Delete this object, used or not

        Cleans up the temporary data sent to the server for dmabuf-based
        :class:`~pywayland.protocol.wayland.WlBuffer` creation.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def add(self, fd: int, plane_idx: int, offset: int, stride: int, modifier_hi: int, modifier_lo: int) -> None:
        """Add a dmabuf to the temporary set

        This request adds one dmabuf to the set in this
        :class:`ZwpLinuxBufferParamsV1`.

        The 64-bit unsigned value combined from modifier_hi and modifier_lo is
        the dmabuf layout modifier. DRM AddFB2 ioctl calls this the fb
        modifier, which is defined in drm_mode.h of Linux UAPI. This is an
        opaque token. Drivers use this token to express tiling, compression,
        etc. driver-specific modifications to the base format defined by the
        DRM fourcc code.

        Starting from version 4, the invalid_format protocol error is sent if
        the format + modifier pair was not advertised as supported.

        Starting from version 5, the invalid_format protocol error is sent if
        all planes don't use the same modifier.

        This request raises the PLANE_IDX error if plane_idx is too large. The
        error PLANE_SET is raised if attempting to set a plane that was already
        set.

        :param fd:
            dmabuf fd
        :type fd:
            `ArgumentType.FileDescriptor`
        :param plane_idx:
            plane index
        :type plane_idx:
            `ArgumentType.Uint`
        :param offset:
            offset in bytes
        :type offset:
            `ArgumentType.Uint`
        :param stride:
            stride in bytes
        :type stride:
            `ArgumentType.Uint`
        :param modifier_hi:
            high 32 bits of layout modifier
        :type modifier_hi:
            `ArgumentType.Uint`
        :param modifier_lo:
            low 32 bits of layout modifier
        :type modifier_lo:
            `ArgumentType.Uint`
        """
        self._marshal(1, fd, plane_idx, offset, stride, modifier_hi, modifier_lo)

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def create(self, width: int, height: int, format: int, flags: int) -> None:
        """Create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given dmabufs

        This asks for creation of a
        :class:`~pywayland.protocol.wayland.WlBuffer` from the added dmabuf
        buffers. The :class:`~pywayland.protocol.wayland.WlBuffer` is not
        created immediately but returned via the 'created' event if the dmabuf
        sharing succeeds. The sharing may fail at runtime for reasons a client
        cannot predict, in which case the 'failed' event is triggered.

        The 'format' argument is a DRM_FORMAT code, as defined by the libdrm's
        drm_fourcc.h. The Linux kernel's DRM sub-system is the authoritative
        source on how the format codes should work.

        The 'flags' is a bitfield of the flags defined in enum "flags".
        'y_invert' means that the image needs to be y-flipped.

        Flag 'interlaced' means that the frame in the buffer is not progressive
        as usual, but interlaced. An interlaced buffer as supported here must
        always contain both top and bottom fields. The top field always begins
        on the first pixel row. The temporal ordering between the two fields is
        top field first, unless 'bottom_first' is specified. It is undefined
        whether 'bottom_first' is ignored if 'interlaced' is not set.

        This protocol does not convey any information about field rate,
        duration, or timing, other than the relative ordering between the two
        fields in one buffer. A compositor may have to estimate the intended
        field rate from the incoming buffer rate. It is undefined whether the
        time of receiving :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` with a new buffer
        attached, applying the :class:`~pywayland.protocol.wayland.WlSurface`
        state, :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` callback trigger,
        presentation, or any other point in the compositor cycle is used to
        measure the frame or field times. There is no support for detecting
        missed or late frames/fields/buffers either, and there is no support
        whatsoever for cooperating with interlaced compositor output.

        The composited image quality resulting from the use of interlaced
        buffers is explicitly undefined. A compositor may use elaborate
        hardware features or software to deinterlace and create progressive
        output frames from a sequence of interlaced input buffers, or it may
        produce substandard image quality. However, compositors that cannot
        guarantee reasonable image quality in all cases are recommended to just
        reject all interlaced buffers.

        Any argument errors, including non-positive width or height, mismatch
        between the number of planes and the format, bad format, bad offset or
        stride, may be indicated by fatal protocol errors: INCOMPLETE,
        INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS.

        Dmabuf import errors in the server that are not obvious client bugs are
        returned via the 'failed' event as non-fatal. This allows attempting
        dmabuf sharing and falling back in the client if it fails.

        This request can be sent only once in the object's lifetime, after
        which the only legal request is destroy. This object should be
        destroyed after issuing a 'create' request. Attempting to use this
        object after issuing 'create' raises the ALREADY_USED protocol error.

        It is not mandatory to issue 'create'. If a client wants to cancel the
        buffer creation, it can just destroy this object.

        :param width:
            base plane width in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            base plane height in pixels
        :type height:
            `ArgumentType.Int`
        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param flags:
            see enum flags
        :type flags:
            `ArgumentType.Uint`
        """
        self._marshal(2, width, height, format, flags)

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.NewId, interface=WlBuffer),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=2,
    )
    def create_immed(self, width: int, height: int, format: int, flags: int) -> WlBufferProxy:
        """Immediately create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given                      dmabufs

        This asks for immediate creation of a
        :class:`~pywayland.protocol.wayland.WlBuffer` by importing the added
        dmabufs.

        In case of import success, no event is sent from the server, and the
        :class:`~pywayland.protocol.wayland.WlBuffer` is ready to be used by
        the client.

        Upon import failure, either of the following may happen, as seen fit by
        the implementation: - the client is terminated with one of the
        following fatal protocol   errors:   - INCOMPLETE, INVALID_FORMAT,
        INVALID_DIMENSIONS, OUT_OF_BOUNDS,     in case of argument errors such
        as mismatch between the number     of planes and the format, bad
        format, non-positive width or     height, or bad offset or stride.   -
        INVALID_WL_BUFFER, in case the cause for failure is unknown or
        platform specific. - the server creates an invalid
        :class:`~pywayland.protocol.wayland.WlBuffer`, marks it as failed and
        sends a 'failed' event to the client. The result of using this
        invalid :class:`~pywayland.protocol.wayland.WlBuffer` as an argument in
        any request by the client is   defined by the compositor
        implementation.

        This takes the same arguments as a 'create' request, and obeys the same
        restrictions.

        :param width:
            base plane width in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            base plane height in pixels
        :type height:
            `ArgumentType.Int`
        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param flags:
            see enum flags
        :type flags:
            `ArgumentType.Uint`
        :returns:
            :class:`~pywayland.protocol.wayland.WlBuffer` -- id for the newly
            created :class:`~pywayland.protocol.wayland.WlBuffer`
        """
        buffer_id = self._marshal_constructor(3, WlBuffer, width, height, format, flags)
        return buffer_id

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.Array),
        version=6,
    )
    def set_sampling_device(self, device: list) -> None:
        """Set the target device of the :class:`~pywayland.protocol.wayland.WlBuffer`

        Set the device the compositor should import the dmabufs to for sampling
        in the next create or create_immed request.

        To avoid race conditions when the compositor removes a device from the
        tranches, it is not a protocol error if the device hasn't been
        advertised by the compositor in a tranche with the sampling flag, but
        the import is likely to fail in that case.

        If the client doesn't know a suitable target device, it shouldn't set
        one, and the compositor should attempt import on all devices it
        supports.

        If the array is too small to contain a dev_t or larger than required,
        the invalid_dev_t_size error will be emitted.

        :param device:
            device dev_t value
        :type device:
            `ArgumentType.Array`
        """
        self._marshal(4, device)


class ZwpLinuxDmabufFeedbackV1Proxy(Proxy[ZwpLinuxDmabufFeedbackV1]):
    interface = ZwpLinuxDmabufFeedbackV1

    @ZwpLinuxDmabufFeedbackV1.request()
    def destroy(self) -> None:
        """Destroy the feedback object

        Using this request a client can tell the server that it is not going to
        use the :class:`ZwpLinuxDmabufFeedbackV1` object anymore.
        """
        self._marshal(0)
        self._destroy()


class ZwpLinuxDmabufV1Proxy(Proxy[ZwpLinuxDmabufV1]):
    interface = ZwpLinuxDmabufV1

    @ZwpLinuxDmabufV1.request()
    def destroy(self) -> None:
        """Unbind the factory

        Objects created through this interface, especially wl_buffers, will
        remain valid.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxBufferParamsV1),
    )
    def create_params(self) -> ZwpLinuxBufferParamsV1Proxy:
        """Create a temporary object for buffer parameters

        This temporary object is used to collect multiple dmabuf handles into a
        single batch to create a :class:`~pywayland.protocol.wayland.WlBuffer`.
        It can only be used once and should be destroyed after a 'created' or
        'failed' event has been received.

        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`
            -- id for the newly created
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`
        """
        params_id = self._marshal_constructor(1, ZwpLinuxBufferParamsV1)
        return params_id

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxDmabufFeedbackV1),
        version=4,
    )
    def get_default_feedback(self) -> ZwpLinuxDmabufFeedbackV1Proxy:
        """Get default feedback

        This request creates a new
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        object not bound to a particular surface. This object will deliver
        feedback about dmabuf parameters to use if the client doesn't support
        per-surface feedback (see get_surface_feedback).

        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        """
        id = self._marshal_constructor(2, ZwpLinuxDmabufFeedbackV1)
        return id

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxDmabufFeedbackV1),
        Argument(ArgumentType.Object, interface=WlSurface),
        version=4,
    )
    def get_surface_feedback(self, surface: WlSurfaceProxy) -> ZwpLinuxDmabufFeedbackV1Proxy:
        """Get feedback for a surface

        This request creates a new
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        object for the specified
        :class:`~pywayland.protocol.wayland.WlSurface`. This object will
        deliver feedback about dmabuf parameters to use for buffers attached to
        this surface.

        If the surface is destroyed before the
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        object, the feedback object becomes inert.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        """
        id = self._marshal_constructor(3, ZwpLinuxDmabufFeedbackV1, surface)
        return id


class ZwpLinuxBufferParamsV1Global(Global[ZwpLinuxBufferParamsV1]):
    interface = ZwpLinuxBufferParamsV1


class ZwpLinuxDmabufFeedbackV1Global(Global[ZwpLinuxDmabufFeedbackV1]):
    interface = ZwpLinuxDmabufFeedbackV1


class ZwpLinuxDmabufV1Global(Global[ZwpLinuxDmabufV1]):
    interface = ZwpLinuxDmabufV1


ZwpLinuxBufferParamsV1._gen_c()
ZwpLinuxBufferParamsV1.proxy_class = ZwpLinuxBufferParamsV1Proxy
ZwpLinuxBufferParamsV1.resource_class = ZwpLinuxBufferParamsV1Resource
ZwpLinuxBufferParamsV1.global_class = ZwpLinuxBufferParamsV1Global


ZwpLinuxDmabufFeedbackV1._gen_c()
ZwpLinuxDmabufFeedbackV1.proxy_class = ZwpLinuxDmabufFeedbackV1Proxy
ZwpLinuxDmabufFeedbackV1.resource_class = ZwpLinuxDmabufFeedbackV1Resource
ZwpLinuxDmabufFeedbackV1.global_class = ZwpLinuxDmabufFeedbackV1Global


ZwpLinuxDmabufV1._gen_c()
ZwpLinuxDmabufV1.proxy_class = ZwpLinuxDmabufV1Proxy
ZwpLinuxDmabufV1.resource_class = ZwpLinuxDmabufV1Resource
ZwpLinuxDmabufV1.global_class = ZwpLinuxDmabufV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import itertools
import xml.etree.ElementTree as ET
from dataclasses import dataclass

from .description import Description
from .element import Element
from .enum import Enum
from .event import Event
from .printer import Printer
from .request import Request


@dataclass(frozen=True)
class Interface(Element):
    name: str
    version: str
    description: Description | None
    enum: list[Enum]
    event: list[Event]
    request: list[Request]

    def __lt__(self, other: Interface) -> bool:
        return self.name < other.name

    @classmethod
    def parse(cls, element: ET.Element) -> Interface:
        """Scanner for interface objects

        Required attributes: `name` and `version`

        Child elements: `description`, `request`, `event`, `enum`
        """
        return cls(
            name=cls.parse_attribute(element, "name"),
            version=cls.parse_attribute(element, "version"),
            description=cls.parse_optional_child(element, Description, "description"),
            enum=cls.parse_repeated_child(element, Enum, "enum"),
            event=cls.parse_repeated_child(element, Event, "event"),
            request=cls.parse_repeated_child(element, Request, "request"),
        )

    @property
    def class_name(self) -> str:
        """Returns the name of the class of the interface

        Camel cases the name of the interface, to be used as the class name.
        """
        return "".join(x.capitalize() for x in self.name.split("_"))

    @property
    def proxy_class_name(self) -> str:
        return f"{self.class_name}Proxy"

    @property
    def resource_class_name(self) -> str:
        return f"{self.class_name}Resource"

    @property
    def global_class_name(self) -> str:
        return f"{self.class_name}Global"

    @property
    def needs_any_type(self) -> bool:
        return any(req.return_type == "Any" for req in self.request)

    @property
    def needs_argument_type(self) -> bool:
        return any(len(method.arg) > 0 for method in self.request + self.event)

    def get_imports(self, all_imports: dict[str, str]) -> set[tuple[str, str]]:
        """Return necessary imports for this interface"""
        return set(
            _import
            for method in itertools.chain(self.request, self.event)
            for _import in method.imports(self.name, all_imports)
        )

    def output_interface(self, printer: Printer) -> None:
        """Generate the output only of the interface class"""
        printer()
        printer()
        # Class definition
        printer(f"class {self.class_name}(Interface):")
        with printer.indented():
            # Docstring
            if self.description:
                self.description.output(printer)
                printer('"""')
                printer()
            # Class attributes
            printer(f'name = "{self.name}"')
            printer(f"version = {self.version}")
            # Enums
            for enum in self.enum:
                printer()
                enum.output(printer)

    def output_requests(self, printer: Printer) -> None:
        """Generate the output only of the proxy class"""
        printer()
        printer()
        printer(f"class {self.proxy_class_name}(Proxy[{self.class_name}]):")
        with printer.indented():
            printer(f"interface = {self.class_name}")
            for opcode, request in enumerate(self.request):
                printer()
                request.output(printer, opcode, self.class_name)

    def output_events(self, printer: Printer) -> None:
        """Generate the output only of the proxy class"""
        printer()
        printer()
        printer(f"class {self.resource_class_name}(Resource[{self.class_name}]):")
        with printer.indented():
            printer(f"interface = {self.class_name}")
            for opcode, event in enumerate(self.event):
                printer()
                event.output(printer, opcode, self.class_name)

    def output_global(self, printer: Printer) -> None:
        """Generate the output only of the proxy class"""
        printer()
        printer()
        printer(f"class {self.global_class_name}(Global[{self.class_name}]):")
        with printer.indented():
            printer(f"interface = {self.class_name}")

    def output_attributes(self, printer: Printer) -> None:
        """Generate the output only of the attributes of the interface"""
        printer()
        printer()
        printer(f"{self.class_name}._gen_c()")
        printer(f"{self.class_name}.proxy_class = {self.proxy_class_name}")
        printer(f"{self.class_name}.resource_class = {self.resource_class_name}")
        printer(f"{self.class_name}.global_class = {self.global_class_name}")



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSeat
from .wayland import WlSeatProxy


class ZwpPrimarySelectionDeviceManagerV1(Interface):
    """X primary selection emulation

    The primary selection device manager is a singleton global object that
    provides access to the primary selection. It allows to create
    wp_primary_selection_source objects, as well as retrieving the per-seat
    wp_primary_selection_device objects.
    """

    name = "zwp_primary_selection_device_manager_v1"
    version = 1


class ZwpPrimarySelectionDeviceV1(Interface):
    name = "zwp_primary_selection_device_v1"
    version = 1


class ZwpPrimarySelectionOfferV1(Interface):
    """Offer to transfer primary selection contents

    A wp_primary_selection_offer represents an offer to transfer the contents
    of the primary selection clipboard to the client. Similar to
    :class:`~pywayland.protocol.wayland.WlDataOffer`, the offer also describes
    the mime types that the data can be converted to and provides the
    mechanisms for transferring the data directly to the client.
    """

    name = "zwp_primary_selection_offer_v1"
    version = 1


class ZwpPrimarySelectionSourceV1(Interface):
    """Offer to replace the contents of the primary selection

    The source side of a wp_primary_selection_offer, it provides a way to
    describe the offered data and respond to requests to transfer the requested
    contents of the primary selection clipboard.
    """

    name = "zwp_primary_selection_source_v1"
    version = 1


class ZwpPrimarySelectionDeviceManagerV1Resource(Resource[ZwpPrimarySelectionDeviceManagerV1]):
    interface = ZwpPrimarySelectionDeviceManagerV1


class ZwpPrimarySelectionDeviceV1Resource(Resource[ZwpPrimarySelectionDeviceV1]):
    interface = ZwpPrimarySelectionDeviceV1

    @ZwpPrimarySelectionDeviceV1.event(
        Argument(ArgumentType.NewId, interface=ZwpPrimarySelectionOfferV1),
    )
    def data_offer(self, offer: ZwpPrimarySelectionOfferV1) -> None:
        """Introduce a new wp_primary_selection_offer

        Introduces a new wp_primary_selection_offer object that may be used to
        receive the current primary selection. Immediately following this
        event, the new wp_primary_selection_offer object will send
        wp_primary_selection_offer.offer events to describe the offered mime
        types.

        :param offer:
        :type offer:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionOfferV1`
        """
        self._post_event(0, offer)

    @ZwpPrimarySelectionDeviceV1.event(
        Argument(ArgumentType.Object, interface=ZwpPrimarySelectionOfferV1, nullable=True),
    )
    def selection(self, id: ZwpPrimarySelectionOfferV1 | None) -> None:
        """Advertise a new primary selection

        The wp_primary_selection_device.selection event is sent to notify the
        client of a new primary selection. This event is sent after the
        wp_primary_selection.data_offer event introducing this object, and
        after the offer has announced its mimetypes through
        wp_primary_selection_offer.offer.

        The data_offer is valid until a new offer or NULL is received or until
        the client loses keyboard focus. The client must destroy the previous
        selection data_offer, if any, upon receiving this event.

        :param id:
        :type id:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionOfferV1` or `None`
        """
        self._post_event(1, id)


class ZwpPrimarySelectionOfferV1Resource(Resource[ZwpPrimarySelectionOfferV1]):
    interface = ZwpPrimarySelectionOfferV1

    @ZwpPrimarySelectionOfferV1.event(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type: str) -> None:
        """Advertise offered mime type

        Sent immediately after creating announcing the
        wp_primary_selection_offer through
        wp_primary_selection_device.data_offer. One event is sent per offered
        mime type.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        """
        self._post_event(0, mime_type)


class ZwpPrimarySelectionSourceV1Resource(Resource[ZwpPrimarySelectionSourceV1]):
    interface = ZwpPrimarySelectionSourceV1

    @ZwpPrimarySelectionSourceV1.event(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def send(self, mime_type: str, fd: int) -> None:
        """Send the primary selection contents

        Request for the current primary selection contents from the client.
        Send the specified mime type over the passed file descriptor, then
        close it.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        :param fd:
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, mime_type, fd)

    @ZwpPrimarySelectionSourceV1.event()
    def cancelled(self) -> None:
        """Request for primary selection contents was canceled

        This primary selection source is no longer valid. The client should
        clean up and destroy this primary selection source.
        """
        self._post_event(1)


class ZwpPrimarySelectionDeviceManagerV1Proxy(Proxy[ZwpPrimarySelectionDeviceManagerV1]):
    interface = ZwpPrimarySelectionDeviceManagerV1

    @ZwpPrimarySelectionDeviceManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPrimarySelectionSourceV1),
    )
    def create_source(self) -> ZwpPrimarySelectionSourceV1Proxy:
        """Create a new primary selection source

        Create a new primary selection source.

        :returns:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionSourceV1`
        """
        id = self._marshal_constructor(0, ZwpPrimarySelectionSourceV1)
        return id

    @ZwpPrimarySelectionDeviceManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPrimarySelectionDeviceV1),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_device(self, seat: WlSeatProxy) -> ZwpPrimarySelectionDeviceV1Proxy:
        """Create a new primary selection device

        Create a new data device for a given seat.

        :param seat:
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionDeviceV1`
        """
        id = self._marshal_constructor(1, ZwpPrimarySelectionDeviceV1, seat)
        return id

    @ZwpPrimarySelectionDeviceManagerV1.request()
    def destroy(self) -> None:
        """Destroy the primary selection device manager

        Destroy the primary selection device manager.
        """
        self._marshal(2)
        self._destroy()


class ZwpPrimarySelectionDeviceV1Proxy(Proxy[ZwpPrimarySelectionDeviceV1]):
    interface = ZwpPrimarySelectionDeviceV1

    @ZwpPrimarySelectionDeviceV1.request(
        Argument(ArgumentType.Object, interface=ZwpPrimarySelectionSourceV1, nullable=True),
        Argument(ArgumentType.Uint),
    )
    def set_selection(self, source: ZwpPrimarySelectionSourceV1Proxy | None, serial: int) -> None:
        """Set the primary selection

        Replaces the current selection. The previous owner of the primary
        selection will receive a wp_primary_selection_source.cancelled event.

        To unset the selection, set the source to NULL.

        :param source:
        :type source:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionSourceV1` or `None`
        :param serial:
            serial of the event that triggered this request
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, source, serial)

    @ZwpPrimarySelectionDeviceV1.request()
    def destroy(self) -> None:
        """Destroy the primary selection device

        Destroy the primary selection device.
        """
        self._marshal(1)
        self._destroy()


class ZwpPrimarySelectionOfferV1Proxy(Proxy[ZwpPrimarySelectionOfferV1]):
    interface = ZwpPrimarySelectionOfferV1

    @ZwpPrimarySelectionOfferV1.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def receive(self, mime_type: str, fd: int) -> None:
        """Request that the data is transferred

        To transfer the contents of the primary selection clipboard, the client
        issues this request and indicates the mime type that it wants to
        receive. The transfer happens through the passed file descriptor
        (typically created with the pipe system call). The source client writes
        the data in the mime type representation requested and then closes the
        file descriptor.

        The receiving client reads from the read end of the pipe until EOF and
        closes its end, at which point the transfer is complete.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        :param fd:
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._marshal(0, mime_type, fd)

    @ZwpPrimarySelectionOfferV1.request()
    def destroy(self) -> None:
        """Destroy the primary selection offer

        Destroy the primary selection offer.
        """
        self._marshal(1)
        self._destroy()


class ZwpPrimarySelectionSourceV1Proxy(Proxy[ZwpPrimarySelectionSourceV1]):
    interface = ZwpPrimarySelectionSourceV1

    @ZwpPrimarySelectionSourceV1.request(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type: str) -> None:
        """Add an offered mime type

        This request adds a mime type to the set of mime types advertised to
        targets. Can be called several times to offer multiple types.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        """
        self._marshal(0, mime_type)

    @ZwpPrimarySelectionSourceV1.request()
    def destroy(self) -> None:
        """Destroy the primary selection source

        Destroy the primary selection source.
        """
        self._marshal(1)
        self._destroy()


class ZwpPrimarySelectionDeviceManagerV1Global(Global[ZwpPrimarySelectionDeviceManagerV1]):
    interface = ZwpPrimarySelectionDeviceManagerV1


class ZwpPrimarySelectionDeviceV1Global(Global[ZwpPrimarySelectionDeviceV1]):
    interface = ZwpPrimarySelectionDeviceV1


class ZwpPrimarySelectionOfferV1Global(Global[ZwpPrimarySelectionOfferV1]):
    interface = ZwpPrimarySelectionOfferV1


class ZwpPrimarySelectionSourceV1Global(Global[ZwpPrimarySelectionSourceV1]):
    interface = ZwpPrimarySelectionSourceV1


ZwpPrimarySelectionDeviceManagerV1._gen_c()
ZwpPrimarySelectionDeviceManagerV1.proxy_class = ZwpPrimarySelectionDeviceManagerV1Proxy
ZwpPrimarySelectionDeviceManagerV1.resource_class = ZwpPrimarySelectionDeviceManagerV1Resource
ZwpPrimarySelectionDeviceManagerV1.global_class = ZwpPrimarySelectionDeviceManagerV1Global


ZwpPrimarySelectionDeviceV1._gen_c()
ZwpPrimarySelectionDeviceV1.proxy_class = ZwpPrimarySelectionDeviceV1Proxy
ZwpPrimarySelectionDeviceV1.resource_class = ZwpPrimarySelectionDeviceV1Resource
ZwpPrimarySelectionDeviceV1.global_class = ZwpPrimarySelectionDeviceV1Global


ZwpPrimarySelectionOfferV1._gen_c()
ZwpPrimarySelectionOfferV1.proxy_class = ZwpPrimarySelectionOfferV1Proxy
ZwpPrimarySelectionOfferV1.resource_class = ZwpPrimarySelectionOfferV1Resource
ZwpPrimarySelectionOfferV1.global_class = ZwpPrimarySelectionOfferV1Global


ZwpPrimarySelectionSourceV1._gen_c()
ZwpPrimarySelectionSourceV1.proxy_class = ZwpPrimarySelectionSourceV1Proxy
ZwpPrimarySelectionSourceV1.resource_class = ZwpPrimarySelectionSourceV1Resource
ZwpPrimarySelectionSourceV1.global_class = ZwpPrimarySelectionSourceV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2024 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpAlphaModifierSurfaceV1(Interface):
    """Alpha modifier object for a surface

    This interface allows the client to set a factor for the alpha values on a
    surface, which can be used to offload such operations to the compositor.
    The default factor is UINT32_MAX.

    This object has to be destroyed before the associated
    :class:`~pywayland.protocol.wayland.WlSurface`. Once the
    :class:`~pywayland.protocol.wayland.WlSurface` is destroyed, all request on
    this object will raise the no_surface error.
    """

    name = "wp_alpha_modifier_surface_v1"
    version = 1

    class error(enum.IntEnum):
        no_surface = 0


class WpAlphaModifierV1(Interface):
    """Surface alpha modifier manager

    This interface allows a client to set a factor for the alpha values on a
    surface, which can be used to offload such operations to the compositor,
    which can in turn for example offload them to KMS.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_alpha_modifier_v1"
    version = 1

    class error(enum.IntEnum):
        already_constructed = 0


class WpAlphaModifierSurfaceV1Resource(Resource[WpAlphaModifierSurfaceV1]):
    interface = WpAlphaModifierSurfaceV1


class WpAlphaModifierV1Resource(Resource[WpAlphaModifierV1]):
    interface = WpAlphaModifierV1


class WpAlphaModifierSurfaceV1Proxy(Proxy[WpAlphaModifierSurfaceV1]):
    interface = WpAlphaModifierSurfaceV1

    @WpAlphaModifierSurfaceV1.request()
    def destroy(self) -> None:
        """Destroy the alpha modifier object

        This destroys the object, and is equivalent to set_multiplier with a
        value of UINT32_MAX, with the same double-buffered semantics as
        set_multiplier.
        """
        self._marshal(0)
        self._destroy()

    @WpAlphaModifierSurfaceV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_multiplier(self, factor: int) -> None:
        """Specify the alpha multiplier

        Sets the alpha multiplier for the surface. The alpha multiplier is
        double-buffered state, see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` for details.

        This factor is applied in the compositor's blending space, as an
        additional step after the processing of per-pixel alpha values for the
        :class:`~pywayland.protocol.wayland.WlSurface`. The exact meaning of
        the factor is thus undefined, unless the blending space is specified in
        a different extension.

        This multiplier is applied even if the buffer attached to the
        :class:`~pywayland.protocol.wayland.WlSurface` doesn't have an alpha
        channel; in that case an alpha value of one is used instead.

        Zero means completely transparent, UINT32_MAX means completely opaque.

        :param factor:
        :type factor:
            `ArgumentType.Uint`
        """
        self._marshal(1, factor)


class WpAlphaModifierV1Proxy(Proxy[WpAlphaModifierV1]):
    interface = WpAlphaModifierV1

    @WpAlphaModifierV1.request()
    def destroy(self) -> None:
        """Destroy the alpha modifier manager object

        Destroy the alpha modifier manager. This doesn't destroy objects
        created with the manager.
        """
        self._marshal(0)
        self._destroy()

    @WpAlphaModifierV1.request(
        Argument(ArgumentType.NewId, interface=WpAlphaModifierSurfaceV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface(self, surface: WlSurfaceProxy) -> WpAlphaModifierSurfaceV1Proxy:
        """Create a new alpha modifier surface object

        Create a new alpha modifier surface object associated with the given
        :class:`~pywayland.protocol.wayland.WlSurface`. If there is already
        such an object associated with the
        :class:`~pywayland.protocol.wayland.WlSurface`, the already_constructed
        error will be raised.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.alpha_modifier_v1.WpAlphaModifierSurfaceV1`
        """
        id = self._marshal_constructor(1, WpAlphaModifierSurfaceV1, surface)
        return id


class WpAlphaModifierSurfaceV1Global(Global[WpAlphaModifierSurfaceV1]):
    interface = WpAlphaModifierSurfaceV1


class WpAlphaModifierV1Global(Global[WpAlphaModifierV1]):
    interface = WpAlphaModifierV1


WpAlphaModifierSurfaceV1._gen_c()
WpAlphaModifierSurfaceV1.proxy_class = WpAlphaModifierSurfaceV1Proxy
WpAlphaModifierSurfaceV1.resource_class = WpAlphaModifierSurfaceV1Resource
WpAlphaModifierSurfaceV1.global_class = WpAlphaModifierSurfaceV1Global


WpAlphaModifierV1._gen_c()
WpAlphaModifierV1.proxy_class = WpAlphaModifierV1Proxy
WpAlphaModifierV1.resource_class = WpAlphaModifierV1Resource
WpAlphaModifierV1.global_class = WpAlphaModifierV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2021 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpTearingControlManagerV1(Interface):
    """Protocol for tearing control

    For some use cases like games or drawing tablets it can make sense to
    reduce latency by accepting tearing with the use of asynchronous page
    flips. This global is a factory interface, allowing clients to inform which
    type of presentation the content of their surfaces is suitable for.

    Graphics APIs like EGL or Vulkan, that manage the buffer queue and commits
    of a :class:`~pywayland.protocol.wayland.WlSurface` themselves, are likely
    to be using this extension internally. If a client is using such an API for
    a :class:`~pywayland.protocol.wayland.WlSurface`, it should not directly
    use this extension on that surface, to avoid raising a
    tearing_control_exists protocol error.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_tearing_control_manager_v1"
    version = 1

    class error(enum.IntEnum):
        tearing_control_exists = 0


class WpTearingControlV1(Interface):
    """Per-surface tearing control interface

    An additional interface to a :class:`~pywayland.protocol.wayland.WlSurface`
    object, which allows the client to hint to the compositor if the content on
    the surface is suitable for presentation with tearing. The default
    presentation hint is vsync. See presentation_hint for more details.

    If the associated :class:`~pywayland.protocol.wayland.WlSurface` is
    destroyed, this object becomes inert and should be destroyed.
    """

    name = "wp_tearing_control_v1"
    version = 1

    class presentation_hint(enum.IntEnum):
        vsync = 0
        async_ = 1


class WpTearingControlManagerV1Resource(Resource[WpTearingControlManagerV1]):
    interface = WpTearingControlManagerV1


class WpTearingControlV1Resource(Resource[WpTearingControlV1]):
    interface = WpTearingControlV1


class WpTearingControlManagerV1Proxy(Proxy[WpTearingControlManagerV1]):
    interface = WpTearingControlManagerV1

    @WpTearingControlManagerV1.request()
    def destroy(self) -> None:
        """Destroy tearing control factory object

        Destroy this tearing control factory object. Other objects, including
        :class:`~pywayland.protocol.tearing_control_v1.WpTearingControlV1`
        objects created by this factory, are not affected by this request.
        """
        self._marshal(0)
        self._destroy()

    @WpTearingControlManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpTearingControlV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_tearing_control(self, surface: WlSurfaceProxy) -> WpTearingControlV1Proxy:
        """Extend surface interface for tearing control

        Instantiate an interface extension for the given
        :class:`~pywayland.protocol.wayland.WlSurface` to request asynchronous
        page flips for presentation.

        If the given :class:`~pywayland.protocol.wayland.WlSurface` already has
        a :class:`~pywayland.protocol.tearing_control_v1.WpTearingControlV1`
        object associated, the tearing_control_exists protocol error is raised.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.tearing_control_v1.WpTearingControlV1`
        """
        id = self._marshal_constructor(1, WpTearingControlV1, surface)
        return id


class WpTearingControlV1Proxy(Proxy[WpTearingControlV1]):
    interface = WpTearingControlV1

    @WpTearingControlV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_presentation_hint(self, hint: int) -> None:
        """Set presentation hint

        Set the presentation hint for the associated
        :class:`~pywayland.protocol.wayland.WlSurface`. This state is double-
        buffered, see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        The compositor is free to dynamically respect or ignore this hint based
        on various conditions like hardware capabilities, surface state and
        user preferences.

        :param hint:
        :type hint:
            `ArgumentType.Uint`
        """
        self._marshal(0, hint)

    @WpTearingControlV1.request()
    def destroy(self) -> None:
        """Destroy tearing control object

        Destroy this surface tearing object and revert the presentation hint to
        vsync. The change will be applied on the next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.
        """
        self._marshal(1)
        self._destroy()


class WpTearingControlManagerV1Global(Global[WpTearingControlManagerV1]):
    interface = WpTearingControlManagerV1


class WpTearingControlV1Global(Global[WpTearingControlV1]):
    interface = WpTearingControlV1


WpTearingControlManagerV1._gen_c()
WpTearingControlManagerV1.proxy_class = WpTearingControlManagerV1Proxy
WpTearingControlManagerV1.resource_class = WpTearingControlManagerV1Resource
WpTearingControlManagerV1.global_class = WpTearingControlManagerV1Global


WpTearingControlV1._gen_c()
WpTearingControlV1.proxy_class = WpTearingControlV1Proxy
WpTearingControlV1.resource_class = WpTearingControlV1Resource
WpTearingControlV1.global_class = WpTearingControlV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import argparse
import logging
import os
import shlex
import subprocess

from .protocol import Protocol

logger = logging.getLogger(__name__)


def pkgconfig(package: str, variable: str) -> str:
    """pkg-config"""
    pkgconfig_env = os.environ.get("PKG_CONFIG", "pkg-config")
    cmd = f"{pkgconfig_env} --variable={variable} {package}"
    args = shlex.split(cmd)
    return subprocess.check_output(args).decode().strip()


def get_wayland_protocols(input_dir: str | None = None) -> list[str]:
    if input_dir:
        protocols_dir = input_dir
    else:
        # use pkg-config to try to find the wayland-protocol directory
        try:
            protocols_dir = pkgconfig("wayland-protocols", "pkgdatadir")
        except subprocess.CalledProcessError:
            raise OSError("Unable to find wayland-protocols using pkgconfig")

    protocols = []
    # wayland protocols stages in incremental importance
    for stages in ("unstable", "staging", "stable"):
        # walk the wayland-protocol dir
        for dirpath, _, filenames in os.walk(os.path.join(protocols_dir, stages)):
            for filename in filenames:
                _, file_ext = os.path.splitext(filename)
                # only generate protocols for xml files
                if file_ext == ".xml":
                    protocols.append(os.path.join(dirpath, filename))
    return protocols


def main() -> None:
    this_dir = os.path.split(__file__)[0]
    protocol_dir = os.path.join(this_dir, "..", "protocol")

    # try to figure out where the wayland.xml file is installed, otherwise use
    # default
    try:
        xml_dir = pkgconfig("wayland-scanner", "pkgdatadir")
        xml_file = os.path.join(xml_dir, "wayland.xml")
    except subprocess.CalledProcessError:
        xml_file = "/usr/share/wayland/wayland.xml"

    parser = argparse.ArgumentParser(
        description="Generate wayland protocol files from xml"
    )
    parser.add_argument(
        "-o",
        "--output-dir",
        metavar="DIR",
        default=protocol_dir,
        type=str,
        help="Directory to output protocol files",
    )
    parser.add_argument(
        "--with-protocols",
        action="store_true",
        help="Also locate and build wayland-protocol xml files (using pkg-config)",
    )
    parser.add_argument(
        "-i",
        "--input",
        metavar="XML_FILE",
        default=[xml_file],
        nargs="+",
        type=str,
        help="Path to input xml file to scan",
    )
    parser.add_argument(
        "--input-protocols",
        metavar="DIR",
        type=str,
        help="Custom directory to wayland-protocol xml files instead of pkg-config",
    )

    args = parser.parse_args()

    if not os.path.exists(args.output_dir):
        os.makedirs(args.output_dir, 0o775)

    input_files = []

    if args.with_protocols:
        protocols_files = get_wayland_protocols(args.input_protocols)
        input_files += protocols_files

    # add wayland.xml at last
    input_files += args.input

    protocols = [Protocol.parse_file(input_file) for input_file in input_files]
    logger.info(f"Parsed {len(protocols)} input xml files")

    # items are created in order of importance (1.stable, 2.staging, 3.unstable)
    # in case duplicates appear in the protocols
    all_imports = {
        interface.name: protocol.name
        for protocol in protocols
        for interface in protocol.interface
    }

    for protocol in protocols:
        # create a new copy so that we can overwrite created interface keys with
        # the interfaces in the current protocol, this is in case an interface name
        # is repeated multiple times fixing the issue of importing and redifining
        # the interface in the same protocol file
        # if a protocol other than the ones with repeated interfaces where to referece
        # the interface name, it would resolve to the interface with highest priority
        # (1.stable, 2.staging, 3.unstable) since the xml files does not have a hard
        # referece to the wanted interface protocol
        current_protocol_imports = all_imports.copy()
        current_protocol_imports.update(
            {interface.name: protocol.name for interface in protocol.interface}
        )
        protocol.output(args.output_dir, current_protocol_imports)
        logger.info(f"Generated protocol: {protocol.name}")


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    main()
